
#line 3 "<stdout>"

#define  YY_INT_ALIGNED short int

/* A lexical scanner generated by flex */

#define FLEX_SCANNER
#define YY_FLEX_MAJOR_VERSION 2
#define YY_FLEX_MINOR_VERSION 5
#define YY_FLEX_SUBMINOR_VERSION 37
#if YY_FLEX_SUBMINOR_VERSION > 0
#define FLEX_BETA
#endif

/* First, we deal with  platform-specific or compiler-specific issues. */

/* begin standard C headers. */
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>

/* end standard C headers. */

/* flex integer type definitions */

#ifndef FLEXINT_H
#define FLEXINT_H

/* C99 systems have <inttypes.h>. Non-C99 systems may or may not. */

#if defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L

/* C99 says to define __STDC_LIMIT_MACROS before including stdint.h,
 * if you want the limit (max/min) macros for int types. 
 */
#ifndef __STDC_LIMIT_MACROS
#define __STDC_LIMIT_MACROS 1
#endif

#include <inttypes.h>
typedef int8_t flex_int8_t;
typedef uint8_t flex_uint8_t;
typedef int16_t flex_int16_t;
typedef uint16_t flex_uint16_t;
typedef int32_t flex_int32_t;
typedef uint32_t flex_uint32_t;
#else
typedef signed char flex_int8_t;
typedef short int flex_int16_t;
typedef int flex_int32_t;
typedef unsigned char flex_uint8_t; 
typedef unsigned short int flex_uint16_t;
typedef unsigned int flex_uint32_t;

/* Limits of integral types. */
#ifndef INT8_MIN
#define INT8_MIN               (-128)
#endif
#ifndef INT16_MIN
#define INT16_MIN              (-32767-1)
#endif
#ifndef INT32_MIN
#define INT32_MIN              (-2147483647-1)
#endif
#ifndef INT8_MAX
#define INT8_MAX               (127)
#endif
#ifndef INT16_MAX
#define INT16_MAX              (32767)
#endif
#ifndef INT32_MAX
#define INT32_MAX              (2147483647)
#endif
#ifndef UINT8_MAX
#define UINT8_MAX              (255U)
#endif
#ifndef UINT16_MAX
#define UINT16_MAX             (65535U)
#endif
#ifndef UINT32_MAX
#define UINT32_MAX             (4294967295U)
#endif

#endif /* ! C99 */

#endif /* ! FLEXINT_H */

#ifdef __cplusplus

/* The "const" storage-class-modifier is valid. */
#define YY_USE_CONST

#else	/* ! __cplusplus */

/* C99 requires __STDC__ to be defined as 1. */
#if defined (__STDC__)

#define YY_USE_CONST

#endif	/* defined (__STDC__) */
#endif	/* ! __cplusplus */

#ifdef YY_USE_CONST
#define uuconst const
#else
#define uuconst
#endif

/* Returned upon end-of-file. */
#define YY_NULL 0

/* Promotes a possibly negative, possibly signed char to an unsigned
 * integer for use as an array index.  If the signed char is negative,
 * we want to instead treat it as an 8-bit unsigned char, hence the
 * double cast.
 */
#define YY_SC_TO_UI(c) ((unsigned int) (unsigned char) c)

/* Enter a start condition.  This macro really ought to take a parameter,
 * but we do it the disgusting crufty way forced on us by the ()-less
 * definition of BEGIN.
 */
#define BEGIN (uu_start) = 1 + 2 *

/* Translate the current start state into a value that can be later handed
 * to BEGIN to return to the state.  The YYSTATE alias is for lex
 * compatibility.
 */
#define YY_START (((uu_start) - 1) / 2)
#define YYSTATE YY_START

/* Action number for EOF rule of a given start state. */
#define YY_STATE_EOF(state) (YY_END_OF_BUFFER + state + 1)

/* Special action meaning "start processing a new file". */
#define YY_NEW_FILE uurestart(uuin  )

#define YY_END_OF_BUFFER_CHAR 0

/* Size of default input buffer. */
#ifndef YY_BUF_SIZE
#define YY_BUF_SIZE 16384
#endif

/* The state buf must be large enough to hold one state per character in the main buffer.
 */
#define YY_STATE_BUF_SIZE   ((YY_BUF_SIZE + 2) * sizeof(uu_state_type))

#ifndef YY_TYPEDEF_YY_BUFFER_STATE
#define YY_TYPEDEF_YY_BUFFER_STATE
typedef struct uu_buffer_state *YY_BUFFER_STATE;
#endif

#ifndef YY_TYPEDEF_YY_SIZE_T
#define YY_TYPEDEF_YY_SIZE_T
typedef size_t uu_size_t;
#endif

extern uu_size_t uuleng;

extern FILE *uuin, *uuout;

#define EOB_ACT_CONTINUE_SCAN 0
#define EOB_ACT_END_OF_FILE 1
#define EOB_ACT_LAST_MATCH 2

    #define YY_LESS_LINENO(n)
    
/* Return all but the first "n" matched characters back to the input stream. */
#define uuless(n) \
	do \
		{ \
		/* Undo effects of setting up uutext. */ \
        int uuless_macro_arg = (n); \
        YY_LESS_LINENO(uuless_macro_arg);\
		*uu_cp = (uu_hold_char); \
		YY_RESTORE_YY_MORE_OFFSET \
		(uu_c_buf_p) = uu_cp = uu_bp + uuless_macro_arg - YY_MORE_ADJ; \
		YY_DO_BEFORE_ACTION; /* set up uutext again */ \
		} \
	while ( 0 )

#define unput(c) uuunput( c, (uutext_ptr)  )

#ifndef YY_STRUCT_YY_BUFFER_STATE
#define YY_STRUCT_YY_BUFFER_STATE
struct uu_buffer_state
	{
	FILE *uu_input_file;

	char *uu_ch_buf;		/* input buffer */
	char *uu_buf_pos;		/* current position in input buffer */

	/* Size of input buffer in bytes, not including room for EOB
	 * characters.
	 */
	uu_size_t uu_buf_size;

	/* Number of characters read into uu_ch_buf, not including EOB
	 * characters.
	 */
	uu_size_t uu_n_chars;

	/* Whether we "own" the buffer - i.e., we know we created it,
	 * and can realloc() it to grow it, and should free() it to
	 * delete it.
	 */
	int uu_is_our_buffer;

	/* Whether this is an "interactive" input source; if so, and
	 * if we're using stdio for input, then we want to use getc()
	 * instead of fread(), to make sure we stop fetching input after
	 * each newline.
	 */
	int uu_is_interactive;

	/* Whether we're considered to be at the beginning of a line.
	 * If so, '^' rules will be active on the next match, otherwise
	 * not.
	 */
	int uu_at_bol;

    int uu_bs_lineno; /**< The line count. */
    int uu_bs_column; /**< The column count. */
    
	/* Whether to try to fill the input buffer when we reach the
	 * end of it.
	 */
	int uu_fill_buffer;

	int uu_buffer_status;

#define YY_BUFFER_NEW 0
#define YY_BUFFER_NORMAL 1
	/* When an EOF's been seen but there's still some text to process
	 * then we mark the buffer as YY_EOF_PENDING, to indicate that we
	 * shouldn't try reading from the input source any more.  We might
	 * still have a bunch of tokens to match, though, because of
	 * possible backing-up.
	 *
	 * When we actually see the EOF, we change the status to "new"
	 * (via uurestart()), so that the user can continue scanning by
	 * just pointing uuin at a new input file.
	 */
#define YY_BUFFER_EOF_PENDING 2

	};
#endif /* !YY_STRUCT_YY_BUFFER_STATE */

/* Stack of input buffers. */
static size_t uu_buffer_stack_top = 0; /**< index of top of stack. */
static size_t uu_buffer_stack_max = 0; /**< capacity of stack. */
static YY_BUFFER_STATE * uu_buffer_stack = 0; /**< Stack as an array. */

/* We provide macros for accessing buffer states in case in the
 * future we want to put the buffer states in a more general
 * "scanner state".
 *
 * Returns the top of the stack, or NULL.
 */
#define YY_CURRENT_BUFFER ( (uu_buffer_stack) \
                          ? (uu_buffer_stack)[(uu_buffer_stack_top)] \
                          : NULL)

/* Same as previous macro, but useful when we know that the buffer stack is not
 * NULL or when we need an lvalue. For internal use only.
 */
#define YY_CURRENT_BUFFER_LVALUE (uu_buffer_stack)[(uu_buffer_stack_top)]

/* uu_hold_char holds the character lost when uutext is formed. */
static char uu_hold_char;
static uu_size_t uu_n_chars;		/* number of characters read into uu_ch_buf */
uu_size_t uuleng;

/* Points to current character in buffer. */
static char *uu_c_buf_p = (char *) 0;
static int uu_init = 0;		/* whether we need to initialize */
static int uu_start = 0;	/* start state number */

/* Flag which is used to allow uuwrap()'s to do buffer switches
 * instead of setting up a fresh uuin.  A bit of a hack ...
 */
static int uu_did_buffer_switch_on_eof;

void uurestart (FILE *input_file  );
void uu_switch_to_buffer (YY_BUFFER_STATE new_buffer  );
YY_BUFFER_STATE uu_create_buffer (FILE *file,int size  );
void uu_delete_buffer (YY_BUFFER_STATE b  );
void uu_flush_buffer (YY_BUFFER_STATE b  );
void uupush_buffer_state (YY_BUFFER_STATE new_buffer  );
void uupop_buffer_state (void );

static void uuensure_buffer_stack (void );
static void uu_load_buffer_state (void );
static void uu_init_buffer (YY_BUFFER_STATE b,FILE *file  );

#define YY_FLUSH_BUFFER uu_flush_buffer(YY_CURRENT_BUFFER )

YY_BUFFER_STATE uu_scan_buffer (char *base,uu_size_t size  );
YY_BUFFER_STATE uu_scan_string (uuconst char *uu_str  );
YY_BUFFER_STATE uu_scan_bytes (uuconst char *bytes,uu_size_t len  );

void *uualloc (uu_size_t  );
void *uurealloc (void *,uu_size_t  );
void uufree (void *  );

#define uu_new_buffer uu_create_buffer

#define uu_set_interactive(is_interactive) \
	{ \
	if ( ! YY_CURRENT_BUFFER ){ \
        uuensure_buffer_stack (); \
		YY_CURRENT_BUFFER_LVALUE =    \
            uu_create_buffer(uuin,YY_BUF_SIZE ); \
	} \
	YY_CURRENT_BUFFER_LVALUE->uu_is_interactive = is_interactive; \
	}

#define uu_set_bol(at_bol) \
	{ \
	if ( ! YY_CURRENT_BUFFER ){\
        uuensure_buffer_stack (); \
		YY_CURRENT_BUFFER_LVALUE =    \
            uu_create_buffer(uuin,YY_BUF_SIZE ); \
	} \
	YY_CURRENT_BUFFER_LVALUE->uu_at_bol = at_bol; \
	}

#define YY_AT_BOL() (YY_CURRENT_BUFFER_LVALUE->uu_at_bol)

/* Begin user sect3 */

typedef unsigned char YY_CHAR;

FILE *uuin = (FILE *) 0, *uuout = (FILE *) 0;

typedef int uu_state_type;

extern int uulineno;

int uulineno = 1;

extern char *uutext;
#define uutext_ptr uutext

static uu_state_type uu_get_previous_state (void );
static uu_state_type uu_try_NUL_trans (uu_state_type current_state  );
static int uu_get_next_buffer (void );
static void uu_fatal_error (uuconst char msg[]  );

/* Done after the current pattern has been matched and before the
 * corresponding action - sets up uutext.
 */
#define YY_DO_BEFORE_ACTION \
	(uutext_ptr) = uu_bp; \
	uuleng = (size_t) (uu_cp - uu_bp); \
	(uu_hold_char) = *uu_cp; \
	*uu_cp = '\0'; \
	(uu_c_buf_p) = uu_cp;

#define YY_NUM_RULES 32
#define YY_END_OF_BUFFER 33
/* This struct is not used in this scanner,
   but its presence is necessary. */
struct uu_trans_info
	{
	flex_int32_t uu_verify;
	flex_int32_t uu_nxt;
	};
static uuconst flex_int16_t uu_acclist[144] =
    {   0,
       33,   31,   32,   31,   32,    5,   32,   31,   32,   31,
       32,   31,   32,   31,   32,   31,   32,   31,   32,   30,
       31,   32,   22,   31,   32,   22,   31,   32,   22,   31,
       32,   31,   32,   31,   32,   31,   32,   31,   32,   22,
       31,   32,   22,   31,   32,   22,   31,   32,   22,   31,
       32,   22,   31,   32,   22,   31,   32,   22,   31,   32,
       22,   31,   32,    4,   17,    3,   18,   30,   26,   26,
       30,   22,   22,   22,   22,   22,    1,    4,    1,    3,
        1,   22,   22,   22,   22,   22,   22,   22,   22,   26,
       26,   27,   28,   29,   22,   22,   22,   22,   22,   22,

       26,   26,   23,   24,16404,   21,   22,   22,   22,    7,
       22,   10,   22,   22,   25, 8212,   19,   22,   22,   22,
        8,   22,   22,   22,   22,    9,   22,   22,   16,   22,
       13,   22,   16,   22,    2,   22,   14,   11,   14,    6,
       15,   12,   15
    } ;

static uuconst flex_int16_t uu_accept[182] =
    {   0,
        1,    1,    1,    1,    1,    2,    4,    6,    8,   10,
       12,   14,   16,   18,   20,   23,   26,   29,   32,   34,
       36,   38,   40,   43,   46,   49,   52,   55,   58,   61,
       64,   64,   64,   64,   64,   65,   65,   66,   66,   67,
       67,   68,   68,   69,   70,   70,   71,   72,   72,   72,
       72,   72,   73,   74,   75,   76,   77,   77,   77,   77,
       77,   79,   79,   81,   81,   82,   83,   84,   85,   86,
       87,   88,   89,   90,   90,   90,   90,   90,   91,   91,
       91,   91,   92,   92,   92,   93,   94,   95,   95,   95,
       95,   95,   96,   97,   98,   99,  100,  101,  102,  102,

      103,  103,  104,  105,  105,  106,  106,  106,  106,  107,
      107,  107,  108,  109,  110,  112,  114,  115,  116,  116,
      117,  117,  118,  118,  118,  118,  118,  119,  120,  121,
      121,  121,  122,  123,  123,  123,  123,  123,  123,  123,
      124,  125,  126,  126,  127,  128,  128,  128,  128,  128,
      128,  129,  131,  133,  135,  135,  135,  135,  135,  135,
      135,  136,  137,  137,  137,  138,  138,  138,  139,  139,
      139,  140,  140,  141,  141,  142,  142,  143,  143,  144,
      144
    } ;

static uuconst flex_int32_t uu_ec[256] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    2,    3,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    2,    4,    5,    6,    1,    1,    1,    7,    8,
        1,    9,   10,    1,   10,   11,    1,   12,   13,   14,
       12,   15,   12,   16,   12,   17,   12,    1,    1,    1,
        1,    1,    1,    1,   18,   19,   20,   21,   22,   23,
       24,   19,   25,   19,   19,   26,   27,   28,   29,   30,
       19,   31,   32,   33,   19,   19,   19,   34,   19,   19,
        1,    1,    1,    1,   35,    1,   18,   19,   36,   37,

       38,   23,   24,   19,   39,   19,   19,   40,   27,   41,
       29,   30,   19,   31,   19,   33,   42,   19,   19,   34,
       19,   43,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,

        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1
    } ;

static uuconst flex_int32_t uu_meta[44] =
    {   0,
        1,    1,    2,    1,    1,    1,    1,    1,    1,    3,
        4,    5,    5,    5,    5,    5,    5,    6,    6,    6,
        6,    6,    6,    6,    6,    6,    6,    6,    6,    6,
        6,    6,    6,    6,    7,    6,    6,    6,    6,    6,
        6,    6,    6
    } ;

static uuconst flex_int16_t uu_base[211] =
    {   0,
        0,   42,   62,  102,  493,  623,   43,  623,  489,  486,
      487,  482,  477,   40,  133,    0,   51,  161,   70,  484,
      480,  477,  200,  200,  450,   30,  448,   37,  241,   32,
       82,  473,  471,  470,  623,  464,  623,  462,  623,  449,
      623,  100,   99,  259,    0,  286,    0,  444,  180,  313,
      147,    0,   39,  423,  411,    0,  228,  423,   42,  444,
      623,  443,  623,  428,  623,  401,  392,  399,  391,   51,
      405,   62,   61,  412,  410,  205,  408,  341,    0,  370,
      211,   89,  407,  387,  623,  623,  623,  374,  242,  376,
      368,  362,  358,   39,   80,   79,   96,   97,  248,  118,

      365,  623,  623,  338,  270,  364,  362,  328,  276,  125,
      329,  325,  126,  341,    0,  179,  280,  623,  317,  298,
      338,  623,  308,  267,  194,  236,  240,  101,  257,  127,
      329,  623,  310,  231,  257,  255,  221,  297,  203,  201,
      197,  198,  357,  623,  311,  351,  210,  179,  279,  151,
      389,  399,  401,  403,  164,  158,  156,  127,  156,  139,
      623,  432,  314,  412,  623,  339,  443,  623,  413,  446,
      137,  120,  623,  449,  623,  453,  623,  469,   93,  623,
      501,  508,  515,  522,  526,  527,  529,  536,  543,  550,
      557,   79,  560,  565,  568,  571,  573,   64,  576,  578,

      581,  584,  587,  590,  593,  596,  599,  605,  609,  615
    } ;

static uuconst flex_int16_t uu_def[211] =
    {   0,
      180,    1,    1,    1,  180,  180,  180,  180,  181,  182,
      183,  184,  185,  186,  180,  187,  187,  187,  180,  188,
      189,  190,  191,  187,  187,   24,  187,  187,  191,   24,
      180,  181,  183,  181,  180,  182,  180,  183,  180,  184,
      180,  180,  185,  180,  192,  193,   15,  194,  180,  180,
      180,  187,   24,   18,   24,   18,  180,  180,  180,  188,
      180,  189,  180,  190,  180,   29,  187,  187,  187,  187,
      187,   29,  187,  195,  196,  180,  197,  180,  198,  198,
      180,  194,  199,   50,  180,  180,  180,  180,  180,  180,
      180,  187,  187,  187,  187,  187,   29,  195,  180,  196,

      180,  180,  180,  180,  180,  200,  201,  180,  180,  180,
      180,  187,  187,  187,  187,  187,   29,  180,  180,  180,
      202,  180,  203,  180,  180,  180,  187,  187,  187,  180,
      180,  180,   29,  180,  204,  205,  180,  180,  180,  187,
      187,  187,  180,  180,   29,  180,  206,  207,  180,  180,
      208,  187,  187,   29,  180,  180,  209,  180,  210,  210,
      180,  208,  180,  180,  180,  180,  180,  180,  190,  190,
      190,  180,  180,  180,  180,  180,  180,  190,  190,    0,
      180,  180,  180,  180,  180,  180,  180,  180,  180,  180,
      180,  180,  180,  180,  180,  180,  180,  180,  180,  180,

      180,  180,  180,  180,  180,  180,  180,  180,  180,  180
    } ;

static uuconst flex_int16_t uu_nxt[667] =
    {   0,
        6,    7,    8,    9,   10,   11,   12,    6,    6,   13,
       14,   15,   15,   15,   15,   15,   15,   16,   16,   16,
       17,   17,   17,   17,   17,   16,   16,   16,   16,   18,
       16,   16,   16,   16,   16,   16,   17,   17,   17,   16,
       16,   16,   16,   19,   31,   20,   32,   21,   33,   89,
       22,   44,   44,   44,   44,   44,   44,   69,   71,   69,
      114,   23,   53,   53,   53,   53,   53,   53,   90,  107,
       69,   57,   73,   32,   71,   33,  114,   23,   95,   52,
       24,   25,   91,   31,   77,   32,   26,   33,   97,   27,
       66,   95,   28,   94,   58,   65,   92,   25,  180,   89,

       26,  115,   27,   19,  116,   20,  180,   21,   59,   46,
       22,   44,   44,   44,   44,   44,   44,  115,  116,   74,
       74,   29,  141,   76,   66,  117,   26,  180,  130,   27,
      122,   76,   28,   51,  131,   74,   74,   29,  141,   65,
       30,  161,   27,   46,   47,   47,   47,   47,   47,   47,
      125,  128,   76,   48,   48,   49,   49,   49,  161,  173,
       85,   86,   50,   87,  125,  128,  122,   51,  105,   48,
       48,   49,   54,   54,   54,   54,   54,   54,   88,  118,
      130,   55,   55,   55,   55,   55,  131,  132,  159,  122,
       56,   83,   83,   83,   83,   83,   83,   55,   55,   55,

       64,   64,   65,   64,   64,   64,   64,   64,   64,   64,
       64,   53,   53,   53,   53,   53,   53,  101,  138,  102,
      105,  103,   98,   98,   98,   98,   98,   98,  153,   57,
      152,   32,  138,   33,   85,   86,  104,   87,  151,  150,
       67,   64,   64,   65,   64,   64,   64,   64,   64,   64,
       64,   64,   58,  109,  109,  109,  109,  109,  109,  100,
      100,  100,  100,  100,  100,  122,   59,  105,  146,   72,
       44,   44,   44,   44,   44,   44,  140,  139,  142,   75,
       75,  120,  120,  120,  120,  120,  120,  109,  109,  109,
      109,  109,  109,   76,  142,   75,   75,   78,   78,   78,

       78,   78,   78,  158,  137,  133,   80,   80,   66,  120,
      120,  120,  120,  120,  120,  163,  149,  158,  122,  133,
       76,  164,   80,   80,   84,   84,   84,   84,   84,   84,
      143,  145,  149,   49,   49,   49,   49,   49,   66,   66,
      166,  144,   50,  144,  154,  144,  167,  145,  105,   49,
       49,   49,   78,   78,   78,   78,   78,   78,  143,  134,
      144,   75,   75,  155,  129,  102,  127,  103,  126,  144,
      124,  144,  122,  144,  105,   76,  119,   75,   75,   99,
      118,  100,  100,  100,  100,  100,  100,  113,  144,  160,
      160,  161,  160,  160,  160,  160,  160,  160,  160,  160,

      163,  112,  166,  111,  169,  110,  164,  165,  167,  168,
      170,  171,  108,  174,  169,   65,  180,   89,  105,   99,
      170,   81,   96,   94,  175,   93,  175,   92,  175,   66,
       65,   66,  160,  160,  161,  160,  160,  160,  160,  160,
      160,  160,  160,  175,  176,   63,   61,  178,   65,   90,
      174,   52,   52,   81,  176,   41,  177,  177,  179,  177,
      179,  175,  179,  175,   39,  175,  177,  177,   37,  177,
      178,   65,   35,   39,  177,   35,   70,  179,   68,   65,
      175,  179,   63,  179,  177,  179,   61,   42,   41,   39,
       37,   35,  180,  180,  180,  180,  180,  180,  180,  180,

      179,   34,   34,   34,   34,   34,   34,   34,   36,  180,
       36,   36,   36,   36,   36,   38,   38,   38,   38,   38,
       38,   38,   40,  180,   40,   40,   40,   40,   40,   43,
       43,   45,   45,   52,   52,   52,   60,   60,   60,   60,
       60,   60,   60,   62,   62,   62,   62,   62,   62,   62,
       64,   64,   64,   64,   64,   64,   64,   66,   66,   66,
       66,   66,   66,   66,   79,   79,   79,   82,  180,   82,
       98,  180,   98,  100,  180,  100,  106,  180,  106,   83,
       83,  121,  180,  121,  123,  180,  123,  135,  180,  135,
      136,  180,  136,  147,  180,  147,  148,  180,  148,  156,

      180,  156,  157,  180,  157,  162,  162,  162,  162,  162,
      162,  162,  172,  180,  172,  160,  160,  160,  160,  160,
      160,  160,    5,  180,  180,  180,  180,  180,  180,  180,
      180,  180,  180,  180,  180,  180,  180,  180,  180,  180,
      180,  180,  180,  180,  180,  180,  180,  180,  180,  180,
      180,  180,  180,  180,  180,  180,  180,  180,  180,  180,
      180,  180,  180,  180,  180,  180
    } ;

static uuconst flex_int16_t uu_chk[667] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    2,    7,    2,    7,    2,    7,   53,
        2,   14,   14,   14,   14,   14,   14,   26,   28,   30,
       94,    2,   17,   17,   17,   17,   17,   17,   59,  198,
       26,   19,   30,   19,   28,   19,   94,    2,   70,   53,
        2,    3,   59,   31,  192,   31,    3,   31,   72,    3,
       72,   70,    3,   73,   19,  179,   73,    3,   82,   82,

        3,   95,    3,    4,   96,    4,   98,    4,   19,   43,
        4,   42,   42,   42,   42,   42,   42,   95,   96,   43,
       43,    4,  128,   82,   97,   97,    4,  100,  130,    4,
      172,   98,    4,   43,  130,   43,   43,    4,  128,  171,
        4,  160,    4,   15,   15,   15,   15,   15,   15,   15,
      110,  113,  100,   15,   15,   15,   15,   15,  159,  158,
       51,   51,   15,   51,  110,  113,  157,   15,  156,   15,
       15,   15,   18,   18,   18,   18,   18,   18,   51,  155,
      116,   18,   18,   18,   18,   18,  116,  116,  150,  148,
       18,   49,   49,   49,   49,   49,   49,   18,   18,   18,

       23,   23,   23,   23,   23,   23,   23,   23,   23,   23,
       23,   24,   24,   24,   24,   24,   24,   76,  125,   76,
      147,   76,   81,   81,   81,   81,   81,   81,  142,   57,
      141,   57,  125,   57,  137,  137,   76,  137,  140,  139,
       24,   29,   29,   29,   29,   29,   29,   29,   29,   29,
       29,   29,   57,   89,   89,   89,   89,   89,   89,   99,
       99,   99,   99,   99,   99,  136,   57,  135,  134,   29,
       44,   44,   44,   44,   44,   44,  127,  126,  129,   44,
       44,  105,  105,  105,  105,  105,  105,  109,  109,  109,
      109,  109,  109,   44,  129,   44,   44,   46,   46,   46,

       46,   46,   46,  149,  124,  117,   46,   46,  117,  120,
      120,  120,  120,  120,  120,  163,  138,  149,  123,  117,
       46,  163,   46,   46,   50,   50,   50,   50,   50,   50,
      131,  133,  138,   50,   50,   50,   50,   50,  133,  145,
      166,  131,   50,  131,  145,  131,  166,  133,  121,   50,
       50,   50,   78,   78,   78,   78,   78,   78,  143,  119,
      131,   78,   78,  146,  114,  146,  112,  146,  111,  143,
      108,  143,  107,  143,  106,   78,  104,   78,   78,   80,
      101,   80,   80,   80,   80,   80,   80,   93,  143,  151,
      151,  151,  151,  151,  151,  151,  151,  151,  151,  151,

      152,   92,  153,   91,  154,   90,  152,  152,  153,  153,
      154,  154,   88,  164,  169,  169,   84,   83,   77,   75,
      169,   74,   71,   69,  164,   68,  164,   67,  164,   66,
       64,  154,  162,  162,  162,  162,  162,  162,  162,  162,
      162,  162,  162,  164,  167,   62,   60,  170,  170,   58,
      174,   55,   54,   48,  176,   40,  167,  167,  170,  167,
      170,  174,  170,  174,   38,  174,  176,  176,   36,  176,
      178,  178,   34,   33,  167,   32,   27,  170,   25,   22,
      174,  178,   21,  178,  176,  178,   20,   13,   12,   11,
       10,    9,    5,    0,    0,    0,    0,    0,    0,    0,

      178,  181,  181,  181,  181,  181,  181,  181,  182,    0,
      182,  182,  182,  182,  182,  183,  183,  183,  183,  183,
      183,  183,  184,    0,  184,  184,  184,  184,  184,  185,
      185,  186,  186,  187,  187,  187,  188,  188,  188,  188,
      188,  188,  188,  189,  189,  189,  189,  189,  189,  189,
      190,  190,  190,  190,  190,  190,  190,  191,  191,  191,
      191,  191,  191,  191,  193,  193,  193,  194,    0,  194,
      195,    0,  195,  196,    0,  196,  197,    0,  197,  199,
      199,  200,    0,  200,  201,    0,  201,  202,    0,  202,
      203,    0,  203,  204,    0,  204,  205,    0,  205,  206,

        0,  206,  207,    0,  207,  208,  208,  208,  208,  208,
      208,  208,  209,    0,  209,  210,  210,  210,  210,  210,
      210,  210,  180,  180,  180,  180,  180,  180,  180,  180,
      180,  180,  180,  180,  180,  180,  180,  180,  180,  180,
      180,  180,  180,  180,  180,  180,  180,  180,  180,  180,
      180,  180,  180,  180,  180,  180,  180,  180,  180,  180,
      180,  180,  180,  180,  180,  180
    } ;

extern int uu_flex_debug;
int uu_flex_debug = 0;

static uu_state_type *uu_state_buf=0, *uu_state_ptr=0;
static char *uu_full_match;
static int uu_lp;
static int uu_looking_for_trail_begin = 0;
static int uu_full_lp;
static int *uu_full_state;
#define YY_TRAILING_MASK 0x2000
#define YY_TRAILING_HEAD_MASK 0x4000
#define REJECT \
{ \
*uu_cp = (uu_hold_char); /* undo effects of setting up uutext */ \
uu_cp = (uu_full_match); /* restore poss. backed-over text */ \
(uu_lp) = (uu_full_lp); /* restore orig. accepting pos. */ \
(uu_state_ptr) = (uu_full_state); /* restore orig. state */ \
uu_current_state = *(uu_state_ptr); /* restore curr. state */ \
++(uu_lp); \
goto find_rule; \
}

#define uumore() uumore_used_but_not_detected
#define YY_MORE_ADJ 0
#define YY_RESTORE_YY_MORE_OFFSET
char *uutext;
#line 1 "nextline.l"
#line 2 "nextline.l"
/*
   Nextline.l is a lex routine which reads Mppl input and converts
   Fortran 90 literal constants to Fortran 77 syntax.  For historical
   reasons, it passes input back to mppl one line at a time.
*/
#include <ctype.h>
#include "scon.h"
#include "mppl.h"

typedef int Integer;

#define CTLVARS FCB_ID(ctlvars, CTLVARS)
#define CINBUF FCB_ID(cinbuf, CINBUF)

#if defined(__osf__) || defined(__hpux)
    typedef unsigned char uutext_t;
#else
    typedef char uutext_t;
#endif

#if defined(sun) && !defined(__SVR4)
int uuwrap(){return 1;}
#else
int uuwrap(void){return 1;}
#endif

/* Keep the loader happy - some lexes stick in a reference to main(), but
   we're calling this stuff from Fortran, and it shows up as an
   unsatisfied reference without the following stub. */
#if defined(__hpux)
static int main(){}
#endif

/* Array of FILE pointers - the Fortran code believes it is handling
   Fortran unit identifiers, but in fact it gets file descriptors,
   which are used to index this array of FILE pointers.  Strictly
   speaking, we should only need MPPL_MAXINLEV+3 elements, but fileno()
   on UNICOS 8 doesn't return contiguous integers.
*/
static FILE *fpa[99];  /* this could probably be [FOPEN_MAX] */
#ifdef FLEX_SCANNER
static YY_BUFFER_STATE include_stack[99];
static int include_stack_ptr = 0;
#endif

/* Following two structs map common blocks from Fortran */
typedef struct {
    Integer lexwarn,
	realsize,
	intsize,
	wordsize,
	inlev,
	finerr,
	nodblquo,
	uuline[MPPL_MAXINLEV];
} ctlvars_t;
FF_EXTERN ctlvars_t CTLVARS;

typedef struct {
    uutext_t cbuf[MPPL_MAXCARD];
} cinbuf_t;
FF_EXTERN cinbuf_t CINBUF;

/* Some short names for particular structure elements */
#define Errcnt CTLVARS.finerr
#define Nodblq CTLVARS.nodblquo
#define Linenum (1+CTLVARS.uuline[CTLVARS.inlev-1])
#define Cbuf CINBUF.cbuf

/* Macros used repeatedly in uulex */
#define Cp		(void)strcpy(cbp,(char *)uutext);cbp+=strlen(uutext)
#define Write(a,b)	(void)sprintf((char *)cbp,a,b);cbp+=strlen(cbp)

/* Functions called by uulex */
static char *modflt(Integer size,Integer wordsize);
static char *modint(Integer size,Integer wordsize);
static void dosquote(void);
static void dodquote(void);

/* Global variables */
static char filename[FILENAMESIZE]; /* The current input filename */
static uutext_t *cbp; /* Pointer onto the string buffer passed from Fortran */


#line 778 "<stdout>"

#define INITIAL 0
#define Im 1

#ifndef YY_NO_UNISTD_H
/* Special case for "unistd.h", since it is non-ANSI. We include it way
 * down here because we want the user's section 1 to have been scanned first.
 * The user has a chance to override it with an option.
 */
#include <unistd.h>
#endif

#ifndef YY_EXTRA_TYPE
#define YY_EXTRA_TYPE void *
#endif

static int uu_init_globals (void );

/* Accessor methods to globals.
   These are made visible to non-reentrant scanners for convenience. */

int uulex_destroy (void );

int uuget_debug (void );

void uuset_debug (int debug_flag  );

YY_EXTRA_TYPE uuget_extra (void );

void uuset_extra (YY_EXTRA_TYPE user_defined  );

FILE *uuget_in (void );

void uuset_in  (FILE * in_str  );

FILE *uuget_out (void );

void uuset_out  (FILE * out_str  );

uu_size_t uuget_leng (void );

char *uuget_text (void );

int uuget_lineno (void );

void uuset_lineno (int line_number  );

/* Macros after this point can all be overridden by user definitions in
 * section 1.
 */

#ifndef YY_SKIP_YYWRAP
#ifdef __cplusplus
extern "C" int uuwrap (void );
#else
extern int uuwrap (void );
#endif
#endif

    static void uuunput (int c,char *buf_ptr  );
    
#ifndef uutext_ptr
static void uu_flex_strncpy (char *,uuconst char *,int );
#endif

#ifdef YY_NEED_STRLEN
static int uu_flex_strlen (uuconst char * );
#endif

#ifndef YY_NO_INPUT

#ifdef __cplusplus
static int uuinput (void );
#else
static int input (void );
#endif

#endif

/* Amount of stuff to slurp up with each read. */
#ifndef YY_READ_BUF_SIZE
#define YY_READ_BUF_SIZE 8192
#endif

/* Copy whatever the last rule matched to the standard output. */
#ifndef ECHO
/* This used to be an fputs(), but since the string might contain NUL's,
 * we now use fwrite().
 */
#define ECHO do { if (fwrite( uutext, uuleng, 1, uuout )) {} } while (0)
#endif

/* Gets input and stuffs it into "buf".  number of characters read, or YY_NULL,
 * is returned in "result".
 */
#ifndef YY_INPUT
#define YY_INPUT(buf,result,max_size) \
	if ( YY_CURRENT_BUFFER_LVALUE->uu_is_interactive ) \
		{ \
		int c = '*'; \
		size_t n; \
		for ( n = 0; n < max_size && \
			     (c = getc( uuin )) != EOF && c != '\n'; ++n ) \
			buf[n] = (char) c; \
		if ( c == '\n' ) \
			buf[n++] = (char) c; \
		if ( c == EOF && ferror( uuin ) ) \
			YY_FATAL_ERROR( "input in flex scanner failed" ); \
		result = n; \
		} \
	else \
		{ \
		errno=0; \
		while ( (result = fread(buf, 1, max_size, uuin))==0 && ferror(uuin)) \
			{ \
			if( errno != EINTR) \
				{ \
				YY_FATAL_ERROR( "input in flex scanner failed" ); \
				break; \
				} \
			errno=0; \
			clearerr(uuin); \
			} \
		}\
\

#endif

/* No semi-colon after return; correct usage is to write "uuterminate();" -
 * we don't want an extra ';' after the "return" because that will cause
 * some compilers to complain about unreachable statements.
 */
#ifndef uuterminate
#define uuterminate() return YY_NULL
#endif

/* Number of entries by which start-condition stack grows. */
#ifndef YY_START_STACK_INCR
#define YY_START_STACK_INCR 25
#endif

/* Report a fatal error. */
#ifndef YY_FATAL_ERROR
#define YY_FATAL_ERROR(msg) uu_fatal_error( msg )
#endif

/* end tables serialization structures and prototypes */

/* Default declaration of generated scanner - a define so the user can
 * easily add parameters.
 */
#ifndef YY_DECL
#define YY_DECL_IS_OURS 1

extern int uulex (void);

#define YY_DECL int uulex (void)
#endif /* !YY_DECL */

/* Code executed at the beginning of each rule, after uutext and uuleng
 * have been set up.
 */
#ifndef YY_USER_ACTION
#define YY_USER_ACTION
#endif

/* Code executed at the end of each rule. */
#ifndef YY_BREAK
#define YY_BREAK break;
#endif

#define YY_RULE_SETUP \
	if ( uuleng > 0 ) \
		YY_CURRENT_BUFFER_LVALUE->uu_at_bol = \
				(uutext[uuleng - 1] == '\n'); \
	YY_USER_ACTION

/** The main scanner function which does all the work.
 */
YY_DECL
{
	register uu_state_type uu_current_state;
	register char *uu_cp, *uu_bp;
	register int uu_act;
    
#line 114 "nextline.l"


#line 967 "<stdout>"

	if ( !(uu_init) )
		{
		(uu_init) = 1;

#ifdef YY_USER_INIT
		YY_USER_INIT;
#endif

        /* Create the reject buffer large enough to save one state per allowed character. */
        if ( ! (uu_state_buf) )
            (uu_state_buf) = (uu_state_type *)uualloc(YY_STATE_BUF_SIZE  );
            if ( ! (uu_state_buf) )
                YY_FATAL_ERROR( "out of dynamic memory in uulex()" );

		if ( ! (uu_start) )
			(uu_start) = 1;	/* first start state */

		if ( ! uuin )
			uuin = stdin;

		if ( ! uuout )
			uuout = stdout;

		if ( ! YY_CURRENT_BUFFER ) {
			uuensure_buffer_stack ();
			YY_CURRENT_BUFFER_LVALUE =
				uu_create_buffer(uuin,YY_BUF_SIZE );
		}

		uu_load_buffer_state( );
		}

	while ( 1 )		/* loops until end-of-file is reached */
		{
		uu_cp = (uu_c_buf_p);

		/* Support of uutext. */
		*uu_cp = (uu_hold_char);

		/* uu_bp points to the position in uu_ch_buf of the start of
		 * the current run.
		 */
		uu_bp = uu_cp;

		uu_current_state = (uu_start);
		uu_current_state += YY_AT_BOL();

		(uu_state_ptr) = (uu_state_buf);
		*(uu_state_ptr)++ = uu_current_state;

uu_match:
		do
			{
			register YY_CHAR uu_c = uu_ec[YY_SC_TO_UI(*uu_cp)];
			while ( uu_chk[uu_base[uu_current_state] + uu_c] != uu_current_state )
				{
				uu_current_state = (int) uu_def[uu_current_state];
				if ( uu_current_state >= 181 )
					uu_c = uu_meta[(unsigned int) uu_c];
				}
			uu_current_state = uu_nxt[uu_base[uu_current_state] + (unsigned int) uu_c];
			*(uu_state_ptr)++ = uu_current_state;
			++uu_cp;
			}
		while ( uu_base[uu_current_state] != 623 );

uu_find_action:
		uu_current_state = *--(uu_state_ptr);
		(uu_lp) = uu_accept[uu_current_state];
find_rule: /* we branch to this label when backing up */
		for ( ; ; ) /* until we find what rule we matched */
			{
			if ( (uu_lp) && (uu_lp) < uu_accept[uu_current_state + 1] )
				{
				uu_act = uu_acclist[(uu_lp)];
				if ( uu_act & YY_TRAILING_HEAD_MASK ||
				     (uu_looking_for_trail_begin) )
					{
					if ( uu_act == (uu_looking_for_trail_begin) )
						{
						(uu_looking_for_trail_begin) = 0;
						uu_act &= ~YY_TRAILING_HEAD_MASK;
						break;
						}
					}
				else if ( uu_act & YY_TRAILING_MASK )
					{
					(uu_looking_for_trail_begin) = uu_act & ~YY_TRAILING_MASK;
					(uu_looking_for_trail_begin) |= YY_TRAILING_HEAD_MASK;
					}
				else
					{
					(uu_full_match) = uu_cp;
					(uu_full_state) = (uu_state_ptr);
					(uu_full_lp) = (uu_lp);
					break;
					}
				++(uu_lp);
				goto find_rule;
				}
			--uu_cp;
			uu_current_state = *--(uu_state_ptr);
			(uu_lp) = uu_accept[uu_current_state];
			}

		YY_DO_BEFORE_ACTION;

do_action:	/* This label is used only to access EOF actions. */

		switch ( uu_act )
	{ /* beginning of action switch */
case 1:
/* rule 1 can match eol */
#line 117 "nextline.l"
case 2:
/* rule 2 can match eol */
#line 118 "nextline.l"
case 3:
/* rule 3 can match eol */
#line 119 "nextline.l"
case 4:
/* rule 4 can match eol */
#line 120 "nextline.l"
case 5:
/* rule 5 can match eol */
YY_RULE_SETUP
#line 120 "nextline.l"
{ Cp;BEGIN 0;return(1); }
	YY_BREAK
case 6:
YY_RULE_SETUP
#line 122 "nextline.l"
{ Cp;BEGIN Im; }
	YY_BREAK
case 7:
YY_RULE_SETUP
#line 123 "nextline.l"
{ Cp; }
	YY_BREAK
case 8:
YY_RULE_SETUP
#line 124 "nextline.l"
{ Cp; }
	YY_BREAK
case 9:
YY_RULE_SETUP
#line 125 "nextline.l"
{ Cp; }
	YY_BREAK
case 10:
YY_RULE_SETUP
#line 126 "nextline.l"
{ Cp;Write("(%d)",CTLVARS.realsize); }
	YY_BREAK
case 11:
YY_RULE_SETUP
#line 127 "nextline.l"
{ Cp; }
	YY_BREAK
case 12:
YY_RULE_SETUP
#line 128 "nextline.l"
{ Cp; }
	YY_BREAK
case 13:
YY_RULE_SETUP
#line 129 "nextline.l"
{ Cp;Write("(%d)",CTLVARS.intsize); }
	YY_BREAK
case 14:
YY_RULE_SETUP
#line 130 "nextline.l"
{ Cp; }
	YY_BREAK
case 15:
YY_RULE_SETUP
#line 131 "nextline.l"
{ Cp; }
	YY_BREAK
case 16:
YY_RULE_SETUP
#line 132 "nextline.l"
{ Cp;Write("(%d)",CTLVARS.realsize); }
	YY_BREAK
case 17:
YY_RULE_SETUP
#line 134 "nextline.l"
{ dodquote(); }
	YY_BREAK
case 18:
YY_RULE_SETUP
#line 135 "nextline.l"
{ dosquote(); }
	YY_BREAK
case 19:
#line 137 "nextline.l"
case 20:
#line 138 "nextline.l"
case 21:
#line 139 "nextline.l"
case 22:
YY_RULE_SETUP
#line 139 "nextline.l"
{ Cp; }
	YY_BREAK
case 23:
YY_RULE_SETUP
#line 141 "nextline.l"
{ Write("%s",modflt(4,CTLVARS.wordsize)); }
	YY_BREAK
case 24:
YY_RULE_SETUP
#line 142 "nextline.l"
{ Write("%s",modflt(8,CTLVARS.wordsize)); }
	YY_BREAK
case 25:
YY_RULE_SETUP
#line 143 "nextline.l"
{ Write("%s",modflt(16,CTLVARS.wordsize)); }
	YY_BREAK
case 26:
YY_RULE_SETUP
#line 144 "nextline.l"
{ Write("%s",modflt(CTLVARS.realsize,CTLVARS.wordsize)); }
	YY_BREAK
case 27:
YY_RULE_SETUP
#line 146 "nextline.l"
{ Write("%s",modint(2,CTLVARS.wordsize)); }
	YY_BREAK
case 28:
YY_RULE_SETUP
#line 147 "nextline.l"
{ Write("%s",modint(4,CTLVARS.wordsize)); }
	YY_BREAK
case 29:
YY_RULE_SETUP
#line 148 "nextline.l"
{ Write("%s",modint(8,CTLVARS.wordsize)); }
	YY_BREAK
case 30:
YY_RULE_SETUP
#line 149 "nextline.l"
{ Write("%s",modint(CTLVARS.intsize,CTLVARS.wordsize)); }
	YY_BREAK
case 31:
YY_RULE_SETUP
#line 151 "nextline.l"
{ *cbp++ = *uutext; }
	YY_BREAK
case 32:
YY_RULE_SETUP
#line 153 "nextline.l"
ECHO;
	YY_BREAK
#line 1224 "<stdout>"
			case YY_STATE_EOF(INITIAL):
			case YY_STATE_EOF(Im):
				uuterminate();

	case YY_END_OF_BUFFER:
		{
		/* Amount of text matched not including the EOB char. */
		int uu_amount_of_matched_text = (int) (uu_cp - (uutext_ptr)) - 1;

		/* Undo the effects of YY_DO_BEFORE_ACTION. */
		*uu_cp = (uu_hold_char);
		YY_RESTORE_YY_MORE_OFFSET

		if ( YY_CURRENT_BUFFER_LVALUE->uu_buffer_status == YY_BUFFER_NEW )
			{
			/* We're scanning a new file or input source.  It's
			 * possible that this happened because the user
			 * just pointed uuin at a new source and called
			 * uulex().  If so, then we have to assure
			 * consistency between YY_CURRENT_BUFFER and our
			 * globals.  Here is the right place to do so, because
			 * this is the first action (other than possibly a
			 * back-up) that will match for the new input source.
			 */
			(uu_n_chars) = YY_CURRENT_BUFFER_LVALUE->uu_n_chars;
			YY_CURRENT_BUFFER_LVALUE->uu_input_file = uuin;
			YY_CURRENT_BUFFER_LVALUE->uu_buffer_status = YY_BUFFER_NORMAL;
			}

		/* Note that here we test for uu_c_buf_p "<=" to the position
		 * of the first EOB in the buffer, since uu_c_buf_p will
		 * already have been incremented past the NUL character
		 * (since all states make transitions on EOB to the
		 * end-of-buffer state).  Contrast this with the test
		 * in input().
		 */
		if ( (uu_c_buf_p) <= &YY_CURRENT_BUFFER_LVALUE->uu_ch_buf[(uu_n_chars)] )
			{ /* This was really a NUL. */
			uu_state_type uu_next_state;

			(uu_c_buf_p) = (uutext_ptr) + uu_amount_of_matched_text;

			uu_current_state = uu_get_previous_state(  );

			/* Okay, we're now positioned to make the NUL
			 * transition.  We couldn't have
			 * uu_get_previous_state() go ahead and do it
			 * for us because it doesn't know how to deal
			 * with the possibility of jamming (and we don't
			 * want to build jamming into it because then it
			 * will run more slowly).
			 */

			uu_next_state = uu_try_NUL_trans( uu_current_state );

			uu_bp = (uutext_ptr) + YY_MORE_ADJ;

			if ( uu_next_state )
				{
				/* Consume the NUL. */
				uu_cp = ++(uu_c_buf_p);
				uu_current_state = uu_next_state;
				goto uu_match;
				}

			else
				{
				uu_cp = (uu_c_buf_p);
				goto uu_find_action;
				}
			}

		else switch ( uu_get_next_buffer(  ) )
			{
			case EOB_ACT_END_OF_FILE:
				{
				(uu_did_buffer_switch_on_eof) = 0;

				if ( uuwrap( ) )
					{
					/* Note: because we've taken care in
					 * uu_get_next_buffer() to have set up
					 * uutext, we can now set up
					 * uu_c_buf_p so that if some total
					 * hoser (like flex itself) wants to
					 * call the scanner after we return the
					 * YY_NULL, it'll still work - another
					 * YY_NULL will get returned.
					 */
					(uu_c_buf_p) = (uutext_ptr) + YY_MORE_ADJ;

					uu_act = YY_STATE_EOF(YY_START);
					goto do_action;
					}

				else
					{
					if ( ! (uu_did_buffer_switch_on_eof) )
						YY_NEW_FILE;
					}
				break;
				}

			case EOB_ACT_CONTINUE_SCAN:
				(uu_c_buf_p) =
					(uutext_ptr) + uu_amount_of_matched_text;

				uu_current_state = uu_get_previous_state(  );

				uu_cp = (uu_c_buf_p);
				uu_bp = (uutext_ptr) + YY_MORE_ADJ;
				goto uu_match;

			case EOB_ACT_LAST_MATCH:
				(uu_c_buf_p) =
				&YY_CURRENT_BUFFER_LVALUE->uu_ch_buf[(uu_n_chars)];

				uu_current_state = uu_get_previous_state(  );

				uu_cp = (uu_c_buf_p);
				uu_bp = (uutext_ptr) + YY_MORE_ADJ;
				goto uu_find_action;
			}
		break;
		}

	default:
		YY_FATAL_ERROR(
			"fatal flex scanner internal error--no action found" );
	} /* end of action switch */
		} /* end of scanning one token */
} /* end of uulex */

/* uu_get_next_buffer - try to read in a new buffer
 *
 * Returns a code representing an action:
 *	EOB_ACT_LAST_MATCH -
 *	EOB_ACT_CONTINUE_SCAN - continue scanning from current position
 *	EOB_ACT_END_OF_FILE - end of file
 */
static int uu_get_next_buffer (void)
{
    	register char *dest = YY_CURRENT_BUFFER_LVALUE->uu_ch_buf;
	register char *source = (uutext_ptr);
	register int number_to_move, i;
	int ret_val;

	if ( (uu_c_buf_p) > &YY_CURRENT_BUFFER_LVALUE->uu_ch_buf[(uu_n_chars) + 1] )
		YY_FATAL_ERROR(
		"fatal flex scanner internal error--end of buffer missed" );

	if ( YY_CURRENT_BUFFER_LVALUE->uu_fill_buffer == 0 )
		{ /* Don't try to fill the buffer, so this is an EOF. */
		if ( (uu_c_buf_p) - (uutext_ptr) - YY_MORE_ADJ == 1 )
			{
			/* We matched a single character, the EOB, so
			 * treat this as a final EOF.
			 */
			return EOB_ACT_END_OF_FILE;
			}

		else
			{
			/* We matched some text prior to the EOB, first
			 * process it.
			 */
			return EOB_ACT_LAST_MATCH;
			}
		}

	/* Try to read more data. */

	/* First move last chars to start of buffer. */
	number_to_move = (int) ((uu_c_buf_p) - (uutext_ptr)) - 1;

	for ( i = 0; i < number_to_move; ++i )
		*(dest++) = *(source++);

	if ( YY_CURRENT_BUFFER_LVALUE->uu_buffer_status == YY_BUFFER_EOF_PENDING )
		/* don't do the read, it's not guaranteed to return an EOF,
		 * just force an EOF
		 */
		YY_CURRENT_BUFFER_LVALUE->uu_n_chars = (uu_n_chars) = 0;

	else
		{
			uu_size_t num_to_read =
			YY_CURRENT_BUFFER_LVALUE->uu_buf_size - number_to_move - 1;

		while ( num_to_read <= 0 )
			{ /* Not enough room in the buffer - grow it. */

			YY_FATAL_ERROR(
"input buffer overflow, can't enlarge buffer because scanner uses REJECT" );

			}

		if ( num_to_read > YY_READ_BUF_SIZE )
			num_to_read = YY_READ_BUF_SIZE;

		/* Read in more data. */
		YY_INPUT( (&YY_CURRENT_BUFFER_LVALUE->uu_ch_buf[number_to_move]),
			(uu_n_chars), num_to_read );

		YY_CURRENT_BUFFER_LVALUE->uu_n_chars = (uu_n_chars);
		}

	if ( (uu_n_chars) == 0 )
		{
		if ( number_to_move == YY_MORE_ADJ )
			{
			ret_val = EOB_ACT_END_OF_FILE;
			uurestart(uuin  );
			}

		else
			{
			ret_val = EOB_ACT_LAST_MATCH;
			YY_CURRENT_BUFFER_LVALUE->uu_buffer_status =
				YY_BUFFER_EOF_PENDING;
			}
		}

	else
		ret_val = EOB_ACT_CONTINUE_SCAN;

	if ((uu_size_t) ((uu_n_chars) + number_to_move) > YY_CURRENT_BUFFER_LVALUE->uu_buf_size) {
		/* Extend the array by 50%, plus the number we really need. */
		uu_size_t new_size = (uu_n_chars) + number_to_move + ((uu_n_chars) >> 1);
		YY_CURRENT_BUFFER_LVALUE->uu_ch_buf = (char *) uurealloc((void *) YY_CURRENT_BUFFER_LVALUE->uu_ch_buf,new_size  );
		if ( ! YY_CURRENT_BUFFER_LVALUE->uu_ch_buf )
			YY_FATAL_ERROR( "out of dynamic memory in uu_get_next_buffer()" );
	}

	(uu_n_chars) += number_to_move;
	YY_CURRENT_BUFFER_LVALUE->uu_ch_buf[(uu_n_chars)] = YY_END_OF_BUFFER_CHAR;
	YY_CURRENT_BUFFER_LVALUE->uu_ch_buf[(uu_n_chars) + 1] = YY_END_OF_BUFFER_CHAR;

	(uutext_ptr) = &YY_CURRENT_BUFFER_LVALUE->uu_ch_buf[0];

	return ret_val;
}

/* uu_get_previous_state - get the state just before the EOB char was reached */

    static uu_state_type uu_get_previous_state (void)
{
	register uu_state_type uu_current_state;
	register char *uu_cp;
    
	uu_current_state = (uu_start);
	uu_current_state += YY_AT_BOL();

	(uu_state_ptr) = (uu_state_buf);
	*(uu_state_ptr)++ = uu_current_state;

	for ( uu_cp = (uutext_ptr) + YY_MORE_ADJ; uu_cp < (uu_c_buf_p); ++uu_cp )
		{
		register YY_CHAR uu_c = (*uu_cp ? uu_ec[YY_SC_TO_UI(*uu_cp)] : 1);
		while ( uu_chk[uu_base[uu_current_state] + uu_c] != uu_current_state )
			{
			uu_current_state = (int) uu_def[uu_current_state];
			if ( uu_current_state >= 181 )
				uu_c = uu_meta[(unsigned int) uu_c];
			}
		uu_current_state = uu_nxt[uu_base[uu_current_state] + (unsigned int) uu_c];
		*(uu_state_ptr)++ = uu_current_state;
		}

	return uu_current_state;
}

/* uu_try_NUL_trans - try to make a transition on the NUL character
 *
 * synopsis
 *	next_state = uu_try_NUL_trans( current_state );
 */
    static uu_state_type uu_try_NUL_trans  (uu_state_type uu_current_state )
{
	register int uu_is_jam;
    
	register YY_CHAR uu_c = 1;
	while ( uu_chk[uu_base[uu_current_state] + uu_c] != uu_current_state )
		{
		uu_current_state = (int) uu_def[uu_current_state];
		if ( uu_current_state >= 181 )
			uu_c = uu_meta[(unsigned int) uu_c];
		}
	uu_current_state = uu_nxt[uu_base[uu_current_state] + (unsigned int) uu_c];
	uu_is_jam = (uu_current_state == 180);
	if ( ! uu_is_jam )
		*(uu_state_ptr)++ = uu_current_state;

		return uu_is_jam ? 0 : uu_current_state;
}

    static void uuunput (int c, register char * uu_bp )
{
	register char *uu_cp;
    
    uu_cp = (uu_c_buf_p);

	/* undo effects of setting up uutext */
	*uu_cp = (uu_hold_char);

	if ( uu_cp < YY_CURRENT_BUFFER_LVALUE->uu_ch_buf + 2 )
		{ /* need to shift things up to make room */
		/* +2 for EOB chars. */
		register uu_size_t number_to_move = (uu_n_chars) + 2;
		register char *dest = &YY_CURRENT_BUFFER_LVALUE->uu_ch_buf[
					YY_CURRENT_BUFFER_LVALUE->uu_buf_size + 2];
		register char *source =
				&YY_CURRENT_BUFFER_LVALUE->uu_ch_buf[number_to_move];

		while ( source > YY_CURRENT_BUFFER_LVALUE->uu_ch_buf )
			*--dest = *--source;

		uu_cp += (int) (dest - source);
		uu_bp += (int) (dest - source);
		YY_CURRENT_BUFFER_LVALUE->uu_n_chars =
			(uu_n_chars) = YY_CURRENT_BUFFER_LVALUE->uu_buf_size;

		if ( uu_cp < YY_CURRENT_BUFFER_LVALUE->uu_ch_buf + 2 )
			YY_FATAL_ERROR( "flex scanner push-back overflow" );
		}

	*--uu_cp = (char) c;

	(uutext_ptr) = uu_bp;
	(uu_hold_char) = *uu_cp;
	(uu_c_buf_p) = uu_cp;
}

#ifndef YY_NO_INPUT
#ifdef __cplusplus
    static int uuinput (void)
#else
    static int input  (void)
#endif

{
	int c;
    
	*(uu_c_buf_p) = (uu_hold_char);

	if ( *(uu_c_buf_p) == YY_END_OF_BUFFER_CHAR )
		{
		/* uu_c_buf_p now points to the character we want to return.
		 * If this occurs *before* the EOB characters, then it's a
		 * valid NUL; if not, then we've hit the end of the buffer.
		 */
		if ( (uu_c_buf_p) < &YY_CURRENT_BUFFER_LVALUE->uu_ch_buf[(uu_n_chars)] )
			/* This was really a NUL. */
			*(uu_c_buf_p) = '\0';

		else
			{ /* need more input */
			uu_size_t offset = (uu_c_buf_p) - (uutext_ptr);
			++(uu_c_buf_p);

			switch ( uu_get_next_buffer(  ) )
				{
				case EOB_ACT_LAST_MATCH:
					/* This happens because uu_g_n_b()
					 * sees that we've accumulated a
					 * token and flags that we need to
					 * try matching the token before
					 * proceeding.  But for input(),
					 * there's no matching to consider.
					 * So convert the EOB_ACT_LAST_MATCH
					 * to EOB_ACT_END_OF_FILE.
					 */

					/* Reset buffer status. */
					uurestart(uuin );

					/*FALLTHROUGH*/

				case EOB_ACT_END_OF_FILE:
					{
					if ( uuwrap( ) )
						return EOF;

					if ( ! (uu_did_buffer_switch_on_eof) )
						YY_NEW_FILE;
#ifdef __cplusplus
					return uuinput();
#else
					return input();
#endif
					}

				case EOB_ACT_CONTINUE_SCAN:
					(uu_c_buf_p) = (uutext_ptr) + offset;
					break;
				}
			}
		}

	c = *(unsigned char *) (uu_c_buf_p);	/* cast for 8-bit char's */
	*(uu_c_buf_p) = '\0';	/* preserve uutext */
	(uu_hold_char) = *++(uu_c_buf_p);

	YY_CURRENT_BUFFER_LVALUE->uu_at_bol = (c == '\n');

	return c;
}
#endif	/* ifndef YY_NO_INPUT */

/** Immediately switch to a different input stream.
 * @param input_file A readable stream.
 * 
 * @note This function does not reset the start condition to @c INITIAL .
 */
    void uurestart  (FILE * input_file )
{
    
	if ( ! YY_CURRENT_BUFFER ){
        uuensure_buffer_stack ();
		YY_CURRENT_BUFFER_LVALUE =
            uu_create_buffer(uuin,YY_BUF_SIZE );
	}

	uu_init_buffer(YY_CURRENT_BUFFER,input_file );
	uu_load_buffer_state( );
}

/** Switch to a different input buffer.
 * @param new_buffer The new input buffer.
 * 
 */
    void uu_switch_to_buffer  (YY_BUFFER_STATE  new_buffer )
{
    
	/* TODO. We should be able to replace this entire function body
	 * with
	 *		uupop_buffer_state();
	 *		uupush_buffer_state(new_buffer);
     */
	uuensure_buffer_stack ();
	if ( YY_CURRENT_BUFFER == new_buffer )
		return;

	if ( YY_CURRENT_BUFFER )
		{
		/* Flush out information for old buffer. */
		*(uu_c_buf_p) = (uu_hold_char);
		YY_CURRENT_BUFFER_LVALUE->uu_buf_pos = (uu_c_buf_p);
		YY_CURRENT_BUFFER_LVALUE->uu_n_chars = (uu_n_chars);
		}

	YY_CURRENT_BUFFER_LVALUE = new_buffer;
	uu_load_buffer_state( );

	/* We don't actually know whether we did this switch during
	 * EOF (uuwrap()) processing, but the only time this flag
	 * is looked at is after uuwrap() is called, so it's safe
	 * to go ahead and always set it.
	 */
	(uu_did_buffer_switch_on_eof) = 1;
}

static void uu_load_buffer_state  (void)
{
    	(uu_n_chars) = YY_CURRENT_BUFFER_LVALUE->uu_n_chars;
	(uutext_ptr) = (uu_c_buf_p) = YY_CURRENT_BUFFER_LVALUE->uu_buf_pos;
	uuin = YY_CURRENT_BUFFER_LVALUE->uu_input_file;
	(uu_hold_char) = *(uu_c_buf_p);
}

/** Allocate and initialize an input buffer state.
 * @param file A readable stream.
 * @param size The character buffer size in bytes. When in doubt, use @c YY_BUF_SIZE.
 * 
 * @return the allocated buffer state.
 */
    YY_BUFFER_STATE uu_create_buffer  (FILE * file, int  size )
{
	YY_BUFFER_STATE b;
    
	b = (YY_BUFFER_STATE) uualloc(sizeof( struct uu_buffer_state )  );
	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in uu_create_buffer()" );

	b->uu_buf_size = size;

	/* uu_ch_buf has to be 2 characters longer than the size given because
	 * we need to put in 2 end-of-buffer characters.
	 */
	b->uu_ch_buf = (char *) uualloc(b->uu_buf_size + 2  );
	if ( ! b->uu_ch_buf )
		YY_FATAL_ERROR( "out of dynamic memory in uu_create_buffer()" );

	b->uu_is_our_buffer = 1;

	uu_init_buffer(b,file );

	return b;
}

/** Destroy the buffer.
 * @param b a buffer created with uu_create_buffer()
 * 
 */
    void uu_delete_buffer (YY_BUFFER_STATE  b )
{
    
	if ( ! b )
		return;

	if ( b == YY_CURRENT_BUFFER ) /* Not sure if we should pop here. */
		YY_CURRENT_BUFFER_LVALUE = (YY_BUFFER_STATE) 0;

	if ( b->uu_is_our_buffer )
		uufree((void *) b->uu_ch_buf  );

	uufree((void *) b  );
}

/* Initializes or reinitializes a buffer.
 * This function is sometimes called more than once on the same buffer,
 * such as during a uurestart() or at EOF.
 */
    static void uu_init_buffer  (YY_BUFFER_STATE  b, FILE * file )

{
	int oerrno = errno;
    
	uu_flush_buffer(b );

	b->uu_input_file = file;
	b->uu_fill_buffer = 1;

    /* If b is the current buffer, then uu_init_buffer was _probably_
     * called from uurestart() or through uu_get_next_buffer.
     * In that case, we don't want to reset the lineno or column.
     */
    if (b != YY_CURRENT_BUFFER){
        b->uu_bs_lineno = 1;
        b->uu_bs_column = 0;
    }

        b->uu_is_interactive = file ? (isatty( fileno(file) ) > 0) : 0;
    
	errno = oerrno;
}

/** Discard all buffered characters. On the next scan, YY_INPUT will be called.
 * @param b the buffer state to be flushed, usually @c YY_CURRENT_BUFFER.
 * 
 */
    void uu_flush_buffer (YY_BUFFER_STATE  b )
{
    	if ( ! b )
		return;

	b->uu_n_chars = 0;

	/* We always need two end-of-buffer characters.  The first causes
	 * a transition to the end-of-buffer state.  The second causes
	 * a jam in that state.
	 */
	b->uu_ch_buf[0] = YY_END_OF_BUFFER_CHAR;
	b->uu_ch_buf[1] = YY_END_OF_BUFFER_CHAR;

	b->uu_buf_pos = &b->uu_ch_buf[0];

	b->uu_at_bol = 1;
	b->uu_buffer_status = YY_BUFFER_NEW;

	if ( b == YY_CURRENT_BUFFER )
		uu_load_buffer_state( );
}

/** Pushes the new state onto the stack. The new state becomes
 *  the current state. This function will allocate the stack
 *  if necessary.
 *  @param new_buffer The new state.
 *  
 */
void uupush_buffer_state (YY_BUFFER_STATE new_buffer )
{
    	if (new_buffer == NULL)
		return;

	uuensure_buffer_stack();

	/* This block is copied from uu_switch_to_buffer. */
	if ( YY_CURRENT_BUFFER )
		{
		/* Flush out information for old buffer. */
		*(uu_c_buf_p) = (uu_hold_char);
		YY_CURRENT_BUFFER_LVALUE->uu_buf_pos = (uu_c_buf_p);
		YY_CURRENT_BUFFER_LVALUE->uu_n_chars = (uu_n_chars);
		}

	/* Only push if top exists. Otherwise, replace top. */
	if (YY_CURRENT_BUFFER)
		(uu_buffer_stack_top)++;
	YY_CURRENT_BUFFER_LVALUE = new_buffer;

	/* copied from uu_switch_to_buffer. */
	uu_load_buffer_state( );
	(uu_did_buffer_switch_on_eof) = 1;
}

/** Removes and deletes the top of the stack, if present.
 *  The next element becomes the new top.
 *  
 */
void uupop_buffer_state (void)
{
    	if (!YY_CURRENT_BUFFER)
		return;

	uu_delete_buffer(YY_CURRENT_BUFFER );
	YY_CURRENT_BUFFER_LVALUE = NULL;
	if ((uu_buffer_stack_top) > 0)
		--(uu_buffer_stack_top);

	if (YY_CURRENT_BUFFER) {
		uu_load_buffer_state( );
		(uu_did_buffer_switch_on_eof) = 1;
	}
}

/* Allocates the stack if it does not exist.
 *  Guarantees space for at least one push.
 */
static void uuensure_buffer_stack (void)
{
	uu_size_t num_to_alloc;
    
	if (!(uu_buffer_stack)) {

		/* First allocation is just for 2 elements, since we don't know if this
		 * scanner will even need a stack. We use 2 instead of 1 to avoid an
		 * immediate realloc on the next call.
         */
		num_to_alloc = 1;
		(uu_buffer_stack) = (struct uu_buffer_state**)uualloc
								(num_to_alloc * sizeof(struct uu_buffer_state*)
								);
		if ( ! (uu_buffer_stack) )
			YY_FATAL_ERROR( "out of dynamic memory in uuensure_buffer_stack()" );
								  
		memset((uu_buffer_stack), 0, num_to_alloc * sizeof(struct uu_buffer_state*));
				
		(uu_buffer_stack_max) = num_to_alloc;
		(uu_buffer_stack_top) = 0;
		return;
	}

	if ((uu_buffer_stack_top) >= ((uu_buffer_stack_max)) - 1){

		/* Increase the buffer to prepare for a possible push. */
		int grow_size = 8 /* arbitrary grow size */;

		num_to_alloc = (uu_buffer_stack_max) + grow_size;
		(uu_buffer_stack) = (struct uu_buffer_state**)uurealloc
								((uu_buffer_stack),
								num_to_alloc * sizeof(struct uu_buffer_state*)
								);
		if ( ! (uu_buffer_stack) )
			YY_FATAL_ERROR( "out of dynamic memory in uuensure_buffer_stack()" );

		/* zero only the new slots.*/
		memset((uu_buffer_stack) + (uu_buffer_stack_max), 0, grow_size * sizeof(struct uu_buffer_state*));
		(uu_buffer_stack_max) = num_to_alloc;
	}
}

/** Setup the input buffer state to scan directly from a user-specified character buffer.
 * @param base the character buffer
 * @param size the size in bytes of the character buffer
 * 
 * @return the newly allocated buffer state object. 
 */
YY_BUFFER_STATE uu_scan_buffer  (char * base, uu_size_t  size )
{
	YY_BUFFER_STATE b;
    
	if ( size < 2 ||
	     base[size-2] != YY_END_OF_BUFFER_CHAR ||
	     base[size-1] != YY_END_OF_BUFFER_CHAR )
		/* They forgot to leave room for the EOB's. */
		return 0;

	b = (YY_BUFFER_STATE) uualloc(sizeof( struct uu_buffer_state )  );
	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in uu_scan_buffer()" );

	b->uu_buf_size = size - 2;	/* "- 2" to take care of EOB's */
	b->uu_buf_pos = b->uu_ch_buf = base;
	b->uu_is_our_buffer = 0;
	b->uu_input_file = 0;
	b->uu_n_chars = b->uu_buf_size;
	b->uu_is_interactive = 0;
	b->uu_at_bol = 1;
	b->uu_fill_buffer = 0;
	b->uu_buffer_status = YY_BUFFER_NEW;

	uu_switch_to_buffer(b  );

	return b;
}

/** Setup the input buffer state to scan a string. The next call to uulex() will
 * scan from a @e copy of @a str.
 * @param uustr a NUL-terminated string to scan
 * 
 * @return the newly allocated buffer state object.
 * @note If you want to scan bytes that may contain NUL values, then use
 *       uu_scan_bytes() instead.
 */
YY_BUFFER_STATE uu_scan_string (uuconst char * uustr )
{
    
	return uu_scan_bytes(uustr,strlen(uustr) );
}

/** Setup the input buffer state to scan the given bytes. The next call to uulex() will
 * scan from a @e copy of @a bytes.
 * @param uubytes the byte buffer to scan
 * @param _uubytes_len the number of bytes in the buffer pointed to by @a bytes.
 * 
 * @return the newly allocated buffer state object.
 */
YY_BUFFER_STATE uu_scan_bytes  (uuconst char * uubytes, uu_size_t  _uubytes_len )
{
	YY_BUFFER_STATE b;
	char *buf;
	uu_size_t n;
	uu_size_t i;
    
	/* Get memory for full buffer, including space for trailing EOB's. */
	n = _uubytes_len + 2;
	buf = (char *) uualloc(n  );
	if ( ! buf )
		YY_FATAL_ERROR( "out of dynamic memory in uu_scan_bytes()" );

	for ( i = 0; i < _uubytes_len; ++i )
		buf[i] = uubytes[i];

	buf[_uubytes_len] = buf[_uubytes_len+1] = YY_END_OF_BUFFER_CHAR;

	b = uu_scan_buffer(buf,n );
	if ( ! b )
		YY_FATAL_ERROR( "bad buffer in uu_scan_bytes()" );

	/* It's okay to grow etc. this buffer, and we should throw it
	 * away when we're done.
	 */
	b->uu_is_our_buffer = 1;

	return b;
}

#ifndef YY_EXIT_FAILURE
#define YY_EXIT_FAILURE 2
#endif

static void uu_fatal_error (uuconst char* msg )
{
    	(void) fprintf( stderr, "%s\n", msg );
	exit( YY_EXIT_FAILURE );
}

/* Redefine uuless() so it works in section 3 code. */

#undef uuless
#define uuless(n) \
	do \
		{ \
		/* Undo effects of setting up uutext. */ \
        int uuless_macro_arg = (n); \
        YY_LESS_LINENO(uuless_macro_arg);\
		uutext[uuleng] = (uu_hold_char); \
		(uu_c_buf_p) = uutext + uuless_macro_arg; \
		(uu_hold_char) = *(uu_c_buf_p); \
		*(uu_c_buf_p) = '\0'; \
		uuleng = uuless_macro_arg; \
		} \
	while ( 0 )

/* Accessor  methods (get/set functions) to struct members. */

/** Get the current line number.
 * 
 */
int uuget_lineno  (void)
{
        
    return uulineno;
}

/** Get the input stream.
 * 
 */
FILE *uuget_in  (void)
{
        return uuin;
}

/** Get the output stream.
 * 
 */
FILE *uuget_out  (void)
{
        return uuout;
}

/** Get the length of the current token.
 * 
 */
uu_size_t uuget_leng  (void)
{
        return uuleng;
}

/** Get the current token.
 * 
 */

char *uuget_text  (void)
{
        return uutext;
}

/** Set the current line number.
 * @param line_number
 * 
 */
void uuset_lineno (int  line_number )
{
    
    uulineno = line_number;
}

/** Set the input stream. This does not discard the current
 * input buffer.
 * @param in_str A readable stream.
 * 
 * @see uu_switch_to_buffer
 */
void uuset_in (FILE *  in_str )
{
        uuin = in_str ;
}

void uuset_out (FILE *  out_str )
{
        uuout = out_str ;
}

int uuget_debug  (void)
{
        return uu_flex_debug;
}

void uuset_debug (int  bdebug )
{
        uu_flex_debug = bdebug ;
}

static int uu_init_globals (void)
{
        /* Initialization is the same as for the non-reentrant scanner.
     * This function is called from uulex_destroy(), so don't allocate here.
     */

    (uu_buffer_stack) = 0;
    (uu_buffer_stack_top) = 0;
    (uu_buffer_stack_max) = 0;
    (uu_c_buf_p) = (char *) 0;
    (uu_init) = 0;
    (uu_start) = 0;

    (uu_state_buf) = 0;
    (uu_state_ptr) = 0;
    (uu_full_match) = 0;
    (uu_lp) = 0;

/* Defined in main.c */
#ifdef YY_STDINIT
    uuin = stdin;
    uuout = stdout;
#else
    uuin = (FILE *) 0;
    uuout = (FILE *) 0;
#endif

    /* For future reference: Set errno on error, since we are called by
     * uulex_init()
     */
    return 0;
}

/* uulex_destroy is for both reentrant and non-reentrant scanners. */
int uulex_destroy  (void)
{
    
    /* Pop the buffer stack, destroying each element. */
	while(YY_CURRENT_BUFFER){
		uu_delete_buffer(YY_CURRENT_BUFFER  );
		YY_CURRENT_BUFFER_LVALUE = NULL;
		uupop_buffer_state();
	}

	/* Destroy the stack itself. */
	uufree((uu_buffer_stack) );
	(uu_buffer_stack) = NULL;

    uufree ( (uu_state_buf) );
    (uu_state_buf)  = NULL;

    /* Reset the globals. This is important in a non-reentrant scanner so the next time
     * uulex() is called, initialization will occur. */
    uu_init_globals( );

    return 0;
}

/*
 * Internal utility routines.
 */

#ifndef uutext_ptr
static void uu_flex_strncpy (char* s1, uuconst char * s2, int n )
{
	register int i;
	for ( i = 0; i < n; ++i )
		s1[i] = s2[i];
}
#endif

#ifdef YY_NEED_STRLEN
static int uu_flex_strlen (uuconst char * s )
{
	register int n;
	for ( n = 0; s[n]; ++n )
		;

	return n;
}
#endif

void *uualloc (uu_size_t  size )
{
	return (void *) malloc( size );
}

void *uurealloc  (void * ptr, uu_size_t  size )
{
	/* The cast to (char *) in the following accommodates both
	 * implementations that use char* generic pointers, and those
	 * that use void* generic pointers.  It works with the latter
	 * because both ANSI C and C++ allow castless assignment from
	 * any pointer type to void*, and deal with argument conversions
	 * as though doing an assignment.
	 */
	return (void *) realloc( (char *) ptr, size );
}

void uufree (void * ptr )
{
	free( (char *) ptr );	/* see uurealloc() for (char *) cast */
}

#define YYTABLES_NAME "uutables"

#line 153 "nextline.l"




/*
    Read input until a newline is seen, copying from uutext into Cbuf.
    Translate literals as you go.  Return -1 in lcbuf for EOF, otherwise
    lcbuf is the linelength, less any trailing whitespace.
*/
#define Nextline FF_ID(nextline, NEXTLINE)
Integer Nextline (Integer *optnum)
{
    extern uutext_t *cbp;
    extern FILE *uuin;
    Integer lcbuf;
    char *s;

    cbp = Cbuf;
    lcbuf = 0;

    if (*optnum == 1) {
       if(uulex()){
	   while(isspace(*--cbp)) {}
   	   lcbuf = 1 + cbp - Cbuf;
       }else{
           lcbuf = -1;
       }
    } else {
       s = fgets(Cbuf, MPPL_MAXCARD, uuin);
       if (s == NULL) {
          lcbuf = -1;
       } else {
          /* don't include the newline */
          lcbuf = strlen(Cbuf) - 1;
       }
    }
    return(lcbuf);
}

/*
    Open the named file.  Return the file descriptor in fd if
    successful.  Store away the FILE pointer and reset uuin to
    the new file.
*/
#define Filopn FF_ID(filopn, FILOPN)
void Filopn (fd,ierr,lname,name)
Integer *fd, *ierr, *lname;
F77_string name;
{
    extern FILE *uuin;
    FILE *fp, *fopen();
    extern char filename[];
    char s[FILENAMESIZE];

    *ierr = -1;
    SC_FORTRAN_STR_C(s,name,*lname);
    if(strcmp(s,"-") == 0){ /* stdin, don't really open it */
	*ierr = 0;
	*fd = 0;
	fpa[*fd] = stdin;
	uuin = stdin;
	(void)strcpy(filename,"STDIN");
    }else if((fp = fopen(s,"r")) != NULL){
	*ierr = 0;
	*fd = fileno(fp);
	fpa[*fd] = fp;
	uuin = fp;
	(void)strcpy(filename,s);
    }
#ifdef FLEX_SCANNER
    if(0 == *ierr){
        include_stack[include_stack_ptr++] = YY_CURRENT_BUFFER;
        uu_switch_to_buffer(uu_create_buffer(uuin,YY_BUF_SIZE ));
    }
#endif
}

/* Close any file except stdin. */
#define Filcls FF_ID(filcls, FILCLS)
void Filcls (fd)
Integer *fd;
{

    if(*fd)fclose(fpa[*fd]);
#ifdef FLEX_SCANNER
    uu_delete_buffer(YY_CURRENT_BUFFER );
#endif
}

/*
    Reset uuin and filename to the previously opened file.
    This function is usually called at the end of
    "include zzuuy" statements.
*/
#define Popyin FF_ID(popyin, POPYIN)
void Popyin (fd,lname,name)
Integer *fd, *lname;
F77_string name;
{
    extern FILE *uuin;
    extern char filename[];

    uuin = fpa[*fd];
    SC_FORTRAN_STR_C(filename,name,*lname);
#ifdef FLEX_SCANNER
    uu_switch_to_buffer(include_stack[--include_stack_ptr] );
#endif
}

/* Modify floating point literal constants appropriately. */
static char *modflt(Integer size,Integer wordsize)
{
	char *underscore = NULL, *expletter = NULL;
	static uutext_t s[128];
        char *t;

        s[0] = '\016';  /* NUMBER from mppl.i */
        s[1] = ' ';
        t = (char *) s + 2;
	(void)strcpy(t,(char *)uutext);
	/* Locate exponent letter and underscore, if present */
        for(; *t != '\0'; t++) {
	    if (isalpha(*t)) {
                expletter = t;
            } else if (*t == '_'){
		underscore = t;
		break;
	    }
	}
#if 0
	Integer i;
	for(i=0; i<uuleng; i++){
	    if(isalpha(s[i])) expletter = (char *) (s + i);
	    if(s[i] == '_'){
		underscore = (char *) (s + i);
		break;
	    }
	}
#endif
	/* First handle constants with 'd' exponent letter */
	if((expletter != NULL) && (*expletter == 'd' || *expletter == 'D')){
	    if(underscore != NULL){
		fprintf(stderr,
                "%s:%d: >>> %s <<< Use 'e' exponent letter instead\n",
		filename,Linenum,uutext);
		++Errcnt;
	    }else if(CTLVARS.lexwarn)
	        fprintf(stderr,
                "%s:%d: Constant size may vary >>> %s <<< Use '_Size' suffix\n",
		filename,Linenum,uutext);
            s[1] = strlen(s)-2;
	    return((char *) s);
	}

	/* Now the remaining constants */

	/* Whack off "_Size" suffix, if present */
	if(underscore != NULL) *underscore = '\0';

	switch(size){
	case 4:
	    /* Nothing to do */
	    break;
	case 8:
	    if(wordsize < 64)
	        if(expletter == NULL) strcat(s,"d0");
	        else *expletter = 'd';
	    break;
	case 16:
	    if(wordsize < 64 && CTLVARS.lexwarn)
		fprintf(stderr,
                "%s:%d: Size16 not available, using Size8 >>> %s <<<\n",
		filename,Linenum,uutext);
	    if(expletter == NULL) strcat(s,"d0");
	    else *expletter = 'd';
	    break;
	default:
	    fprintf(stderr,"%s:%d >>> %s <<< Unknown Size Specifier\n",
	    filename,Linenum,uutext);
	    ++Errcnt;
	    break;
	}

        s[1] = strlen(s)-2;
	return((char *) s);
}

/* Modify integer literal constants appropriately. */
static char *modint(Integer size,Integer wordsize)
{
	char *underscore = NULL;
        char *t;
	static uutext_t s[128];

	(void)strcpy(s,(char *)uutext);
	/* Locate underscore, if present */
	for(t = (char *) s; *t != '\0'; t++){
	    if(*t == '_'){
		underscore = t;
		break;
	    }
	}

	/* Whack off "_Size" suffix, if present */
	if(underscore != NULL) *underscore = '\0';

	switch(size){
	case 2:
	    /* Nothing to do */
	    break;
	case 4:
	    /* Also nothing to do */
	    break;
	case 8:
	    if(wordsize < 64 && CTLVARS.lexwarn)
		fprintf(stderr,
                "%s:%d: Size8 not available, using Size4 >>> %s <<<\n",
		filename,Linenum,uutext);
	    break;
	default:
	    fprintf(stderr,
	    "%s:%d >>> %s <<< Unknown Size Specifier\n",
	    filename,Linenum,uutext);
	    ++Errcnt;
	    break;
	}
	return((char *) s);
}

/*
   Change double-quoted things to single-quoted things for Fortran.
   Yytext has a double-quoted string in it.  Internal single quotes
   get doubled; double quotes are converted to single.
*/
static void dodquote(void)
{
    extern uutext_t *cbp;
    uutext_t *yp;

    if(!Nodblq){
	Cp;
	return;
    }
    for(yp=uutext;*yp;yp++){
	switch(*yp){
	case '\"': *cbp++ = '\''; break;
	case '\'': *cbp++ = '\''; *cbp++ = '\''; break;
	default: *cbp++ = *yp;
	}
    }
}

/*
   Change single-quoted things to single-quoted things for Fortran.
   (They may have double quotes in the interior of the string,
   which get changed to a pair of single quotes.)
*/
static void dosquote(void)
{
    extern uutext_t *cbp;
    uutext_t *yp;

    if(!Nodblq){
	Cp;
	return;
    }
    for(yp=uutext;*yp;yp++){
	switch(*yp){
	case '\"': *cbp++ = '\''; *cbp++ = '\''; break;
	default: *cbp++ = *yp;
	}
    }
}

